@@ -29,13 +29,11 @@
 #include <kdebug.h>
 #include <QList>
 #include <QRectF>
-
 #include <KoGenStyle.h>
 
 #include "document.h"
 #include "texthandler.h"
 
-
 KWordTableHandler::KWordTableHandler(KoXmlWriter* bodyWriter, KoGenStyles* mainStyles)
 {
     // This strange value (-2), is used to create a check that e.g.  a
@@ -70,24 +68,32 @@ void KWordTableHandler::tableStart(KWord::Table* table)
     writer->startElement("table:table");
 
     // Start a table style.
-    KoGenStyle tableStyle(KoGenStyle::StyleAutoTable, "table");
+    KoGenStyle tableStyle(KoGenStyle::TableAutoStyle, "table");
     tableStyle.addPropertyPt("style:width", (table->m_cellEdges[table->m_cellEdges.size()-1] - table->m_cellEdges[0]) / 20.0);
     tableStyle.addProperty("style:border-model", "collapsing");
-    QString tableStyleName = m_mainStyles->lookup(tableStyle, QString("Table"), KoGenStyles::AllowDuplicates);
+
+    // Check to see if we need a master page name attribute.
+    if (document()->writeMasterStyleName()) {
+      tableStyle.addAttribute("style:master-page-name",
+			      document()->masterStyleName());
+      document()->set_writeMasterStyleName(false);
+    }
+
+    QString tableStyleName = m_mainStyles->insert(tableStyle, QLatin1String("Table"), KoGenStyles::AllowDuplicates);
 
     writer->addAttribute("table:style-name", tableStyleName);
 
     // Write the table:table-column descriptions.
     for (int r = 0; r < table->m_cellEdges.size() - 1; r++) {
-        KoGenStyle tableColumnStyle(KoGenStyle::StyleAutoTableColumn, "table-column");
+        KoGenStyle tableColumnStyle(KoGenStyle::TableColumnAutoStyle, "table-column");
         tableColumnStyle.addPropertyPt("style:column-width",
                                        (table->m_cellEdges[r+1] - table->m_cellEdges[r]) / 20.0);
 
         QString tableColumnStyleName;
         if (r >= 26)
-            tableColumnStyleName = m_mainStyles->lookup(tableColumnStyle, tableStyleName + ".A" + QChar('A' + r - 26), KoGenStyles::DontForceNumbering);
+            tableColumnStyleName = m_mainStyles->insert(tableColumnStyle, tableStyleName + ".A" + QChar('A' + r - 26), KoGenStyles::DontAddNumberToName);
         else
-            tableColumnStyleName = m_mainStyles->lookup(tableColumnStyle, tableStyleName + '.' + QChar('A' + r), KoGenStyles::DontForceNumbering);
+            tableColumnStyleName = m_mainStyles->insert(tableColumnStyle, tableStyleName + '.' + QChar('A' + r), KoGenStyles::DontAddNumberToName);
 
         writer->startElement("table:table-column");
         writer->addAttribute("table:style-name", tableColumnStyleName);
@@ -120,8 +126,7 @@ void KWordTableHandler::tableRowStart(wvWare::SharedPtr<const wvWare::Word97::TA
     //kDebug(30513) << "tableRowStart row=" << m_row
     //            << ", number of cells: " << tap->itcMac;
 
-    KoGenStyle rowStyle(KoGenStyle::StyleAutoTableRow, "table-row");
-    QString rowStyleName = m_mainStyles->lookup(rowStyle, QString("row"));
+    KoGenStyle rowStyle(KoGenStyle::TableRowAutoStyle, "table-row");
 
     // The 6 BRC objects are for top, left, bottom, right,
     // insidehorizontal, insidevertical (default values).
@@ -133,16 +138,21 @@ void KWordTableHandler::tableRowStart(wvWare::SharedPtr<const wvWare::Word97::TA
     }
     // We ignore brc.dptSpace (spacing), brc.fShadow (shadow), and brc.fFrame (?)
 
+    qreal rowHeightPt = Conversion::twipsToPt(qAbs(tap->dyaRowHeight));         // convert twips to Pts
+    QString rowHeightString = QString::number(rowHeightPt).append("pt");        // make height string from number
+
     if (tap->dyaRowHeight > 0) {
-        rowStyle.addProperty("style:min-row-height", tap->dyaRowHeight);
-    } else if (tap->dyaRowHeight > 0) {
-        rowStyle.addProperty("style:row-height", -tap->dyaRowHeight);
+        rowStyle.addProperty("style:min-row-height", rowHeightString);
+    } else if (tap->dyaRowHeight < 0) {
+        rowStyle.addProperty("style:row-height", rowHeightString);
     }
 
     if (tap->fCantSplit) {
         rowStyle.addProperty("style:keep-together", "always");
     }
 
+    QString rowStyleName = m_mainStyles->insert(rowStyle, QLatin1String("row"));
+
     //start table row in content
     writer->startElement("table:table-row");
     writer->addAttribute("table:style-name", rowStyleName.toUtf8());
@@ -159,8 +169,8 @@ void KWordTableHandler::tableRowEnd()
 
 KoXmlWriter * KWordTableHandler::currentWriter()
 {
-    if (document()->textHandler()->m_writingHeader && document()->textHandler()->m_headerWriter != NULL)
-        return document()->textHandler()->m_headerWriter;
+    if (document()->writingHeader() && document()->headerWriter() != NULL)
+        return document()->headerWriter();
     else
         return m_bodyWriter;
 }
@@ -335,7 +345,7 @@ void KWordTableHandler::tableCellStart()
                                           brcWinner(tc.brcRight, m_tap->rgbrcTable[5]) :
                                           brcWinner(tc.brcRight, m_tap->rgbrcTable[3]);
 
-    KoGenStyle cellStyle(KoGenStyle::StyleAutoTableCell, "table-cell");
+    KoGenStyle cellStyle(KoGenStyle::TableCellAutoStyle, "table-cell");
     //set borders for the four edges of the cell
     if (brcTop.brcType > 0 && brcTop.brcType < 64) {
         cellStyle.addProperty("fo:border-top", Conversion::setBorderAttributes(brcTop));
@@ -372,6 +382,11 @@ void KWordTableHandler::tableCellStart()
     if (shd.ipat == 0 && shd.cvBack != 0xff000000)
         cellStyle.addProperty("fo:background-color", '#' + QString::number(shd.cvBack | 0xff000000, 16).right(6).toUpper());
 
+    if (shd.ipat >= 0x02 && shd.ipat <=0x0d) {                          // if it's ipatPct5 to ipatPct90
+        uint grayColor = Conversion::shadingPatternToColor(shd.ipat);   // get the color from shading pattern
+        cellStyle.addProperty("fo:background-color", '#' + QString::number(grayColor | 0xff000000, 16).right(6).toUpper());
+    }
+
     //text direction
     //if(tc.fVertical) {
     //    cellStyle.addProperty("style:direction", "ttb");
@@ -386,7 +401,7 @@ void KWordTableHandler::tableCellStart()
         cellStyle.addProperty("style:vertical-align", "bottom");
     }
 
-    QString cellStyleName = m_mainStyles->lookup(cellStyle, QString("cell"));
+    QString cellStyleName = m_mainStyles->insert(cellStyle, QLatin1String("cell"));
 
     //emit sigTableCellStart( m_row, leftCellNumber, rowSpan, colSpan, cellRect, m_currentTable->name, brcTop, brcBottom, brcLeft, brcRight, m_tap->rgshd[ m_column ] );
 
