@@ -32,8 +32,9 @@
 #include <wv2/src/lists.h>
 #include <QString>
 #include <QObject>
-#include <qdom.h>
+#include <QDomElement>
 #include <QBuffer>
+#include <QStack>
 
 #include <KoXmlWriter.h>
 #include <KoGenStyles.h>
@@ -74,8 +75,12 @@ public:
     virtual void sectionStart(wvWare::SharedPtr<const wvWare::Word97::SEP> sep);
     virtual void sectionEnd();
     virtual void headersFound(const wvWare::HeaderFunctor& parseHeaders);
-    virtual void footnoteFound(wvWare::FootnoteData::Type type, wvWare::UChar character,
-                               wvWare::SharedPtr<const wvWare::Word97::CHP> chp, const wvWare::FootnoteFunctor& parseFootnote);
+    virtual void footnoteFound(wvWare::FootnoteData::Type type, wvWare::UString characters,
+                               wvWare::SharedPtr<const wvWare::Word97::CHP> chp,
+                               const wvWare::FootnoteFunctor& parseFootnote);
+    virtual void annotationFound(wvWare::UString characters,
+                                 wvWare::SharedPtr<const wvWare::Word97::CHP> chp,
+                                 const wvWare::AnnotationFunctor& parseAnnotation);
 
     virtual void paragraphStart(wvWare::SharedPtr<const wvWare::ParagraphProperties> paragraphProperties);
     virtual void paragraphEnd();
@@ -90,6 +95,7 @@ public:
     virtual void pictureFound(const wvWare::PictureFunctor& picture, wvWare::SharedPtr<const wvWare::Word97::PICF> picf,
                               wvWare::SharedPtr<const wvWare::Word97::CHP> chp);
 #endif // IMAGE_IMPORT
+    void drawingFound(unsigned int globalCP );
     ///////// Our own interface, also used by processStyles
 
     Document* document() const {
@@ -106,13 +112,9 @@ public:
     // Write the _contents_ (children) of a <LAYOUT> or <STYLE> tag, from the given parag props
     //void writeLayout(const wvWare::ParagraphProperties& paragraphProperties, KoGenStyle* paragraphStyle, const wvWare::Style* style, bool writeContentTags, QString namedStyle);
 
-    bool m_writingHeader; //flag for headers & footers, where we write the actual text to styles.xml
-    bool m_writeMasterStyleName; //whether to write the style name or not, since it only needs to be the first one
     bool listIsOpen(); //tell us whether a list is open
     void closeList();
-    KoXmlWriter* m_headerWriter; //for header/footer writing in styles.xml
     QString m_listStyleName; //track the name of the list style
-    QString m_masterStyleName; //need to know what the master style name is so we can write it
     KoGenStyles* m_mainStyles; //this is for collecting most of the styles
     int m_sectionNumber;
     QString getFont(unsigned fc) const;
@@ -123,10 +125,12 @@ signals:
     void sectionEnd(wvWare::SharedPtr<const wvWare::Word97::SEP>);
     void subDocFound(const wvWare::FunctorBase* parsingFunctor, int data);
     void footnoteFound(const wvWare::FunctorBase* parsingFunctor, int data);
+    void annotationFound(const wvWare::FunctorBase* parsingFunctor, int data);
     void headersFound(const wvWare::FunctorBase* parsingFunctor, int data);
     void tableFound(KWord::Table* table);
     void pictureFound(const QString& frameName, const QString& pictureName, KoXmlWriter* writer,
                       const wvWare::FunctorBase* pictureFunctor);
+    void drawingFound(unsigned int globalCP, KoXmlWriter* writer);
     void updateListDepth(int);
 
 protected:
@@ -168,6 +172,7 @@ private:
     void saveState();
     void restoreState();
 
+    QStack <KoXmlWriter*> m_usedListWriters;
 
     // Current paragraph
     wvWare::SharedPtr<const wvWare::Word97::SEP> m_sep; //store section info for section end
@@ -185,9 +190,18 @@ private:
     bool m_fieldAfterSeparator;
     int m_fieldType; //0 if we're not in a field, -1 for a field we can't handle,
     //anything else is the type of the field
+
     bool m_insideFootnote;
     KoXmlWriter* m_footnoteWriter; //write the footnote data, then add it to bodyWriter
     QBuffer* m_footnoteBuffer; //buffer for the footnote data
+
+    bool m_insideAnnotation;
+    KoXmlWriter* m_annotationWriter; //write the annotation data, then add it to bodyWriter
+    QBuffer* m_annotationBuffer; //buffer for the annotation data
+
+    bool m_insideDrawing;
+    KoXmlWriter* m_drawingWriter; //write the drawing data, then add it to bodyWriter
+
     int m_maxColumns;//max number of columns in a table
 
     bool writeListInfo(KoXmlWriter* writer, const wvWare::Word97::PAP& pap, const wvWare::ListInfo* listInfo);
@@ -197,6 +211,7 @@ private:
     QString m_previousListStyleName;
 
     QList<QString> m_hyperLinkList;
+    bool m_hyperLinkActive;
 };
 
 #endif // TEXTHANDLER_H
