@@ -44,6 +44,7 @@ namespace MSOOXML
 class MsooXmlReader;
 class MsooXmlReaderContext;
 class MsooXmlRelationships;
+class DrawingMLTheme;
 
 //! A base class for MSOOXML-to-ODF import filters
 class MSOOXML_EXPORT MsooXmlImport : public KoOdfExporter
@@ -54,6 +55,8 @@ public:
 
     virtual ~MsooXmlImport();
 
+    KoStore* outputStore() const { return m_outputStore; }
+
     //! KoFilter::UsageError is returned if this method is called outside
     //! of the importing process, i.e. not from within parseParts().
     KoFilter::ConversionStatus loadAndParseDocument(MsooXmlReader *reader, const QString& path,
@@ -92,6 +95,9 @@ protected:
         const QByteArray& contentType, MsooXmlReader *reader, KoOdfWriters *writers,
         QString& errorMessage, MsooXmlReaderContext* context = 0);
 
+    KoFilter::ConversionStatus parseThemes(QMap<QString, MSOOXML::DrawingMLTheme*>& themes,
+        KoOdfWriters *writers, QString& errorMessage);
+
 private:
     //! Opens file for converting and performs convertions.
     //! @return status of convertion.
