@@ -1,7 +1,7 @@
 /* This file is part of the KDE project
 
    Copyright 2007 Stefan Nikolaus     <stefan.nikolaus@kdemail.net>
-   Copyright 2007-2009 Inge Wallin    <inge@lysator.liu.se>
+   Copyright 2007-2010 Inge Wallin    <inge@lysator.liu.se>
    Copyright 2007-2008 Johannes Simon <johannes.simon@gmail.com>
 
    This library is free software; you can redistribute it and/or
@@ -89,6 +89,7 @@
 #include <KoShapeBorderModel.h>
 #include <KoColorBackground.h>
 #include <KoLineBorder.h>
+#include <KoOdfWorkaround.h>
 
 // KChart
 #include "Axis.h"
@@ -163,7 +164,7 @@ QString saveOdfFont( KoGenStyles& mainStyles,
                      const QColor& color )
 {
     KoGenStyle::PropertyType tt = KoGenStyle::TextType;
-    KoGenStyle autoStyle( KoGenStyle::StyleAuto, "chart", 0 );
+    KoGenStyle autoStyle( KoGenStyle::ParagraphAutoStyle, "chart", 0 );
     autoStyle.addProperty( "fo:font-family", font.family(), tt );
     autoStyle.addPropertyPt( "fo:font-size", font.pointSize(), tt );
     autoStyle.addProperty( "fo:color", color.isValid() ? color.name() : "#000000", tt );
@@ -171,7 +172,7 @@ QString saveOdfFont( KoGenStyles& mainStyles,
     autoStyle.addProperty( "fo:font-weight", w == 50 ? "normal" : w == 75 ? "bold" : QString::number( qRound(  w / 10 ) * 100 ), tt );
     autoStyle.addProperty( "fo:font-style", font.italic() ? "italic" : "normal", tt );
 
-    return mainStyles.lookup( autoStyle, "ch", KoGenStyles::ForceNumbering );
+    return mainStyles.insert( autoStyle, "ch" );
 }
 
 bool loadOdfLabel( KoShape *label, KoXmlElement &labelElement, KoShapeLoadingContext &context )
@@ -180,7 +181,9 @@ bool loadOdfLabel( KoShape *label, KoXmlElement &labelElement, KoShapeLoadingCon
     if ( !labelData )
         return false;
 
-    label->loadOdf( labelElement, context );
+    // Following will always return false cause KoTextShapeData::loadOdf will try to load
+    // a frame while our text:p is not within a frame. So, let's just not call loadOdf then...
+    //label->loadOdf( labelElement, context );
 
     // TODO: Read optional attributes
     // 1. Table range
@@ -189,8 +192,6 @@ bool loadOdfLabel( KoShape *label, KoXmlElement &labelElement, KoShapeLoadingCon
 
     labelData->document()->setPlainText( pElement.text() );
 
-    label->setVisible( true );
-
     return true;
 }
 
@@ -260,7 +261,9 @@ public:
     Private( ChartShape *shape );
     ~Private();
 
-    void showLabel( KoShape *label );
+    void showLabel( KoShape *label, bool doShow );
+    QPointF relativePosition( KoShape *shape );
+    void setRelativePosition( KoShape *shape, const QPointF &relPos );
 
     // The components of a chart
     KoShape   *title;
@@ -277,11 +280,12 @@ public:
 
     ChartShape *shape;		// The chart that owns this ChartShape::Private
 
-    QMap<QString, KoDataCenter*> dataCenterMap;
+    KoResourceManager *resourceManager;
 };
 
 
 ChartShape::Private::Private( ChartShape *shape )
+    : resourceManager(0)
 {
     // Register the owner.
     this->shape = shape;
@@ -310,56 +314,50 @@ ChartShape::Private::~Private()
 //
 // If there is too little room, then make space by shrinking the Plotarea.
 //
-// FIXME: Make it take a bool parameter to show / hide the label in
-//        question and then also grow the Plotarea again if the label
-//        is hidden.
-void ChartShape::Private::showLabel( KoShape *label )
+void ChartShape::Private::showLabel( KoShape *label, bool doShow )
 {
     Q_ASSERT( label );
 
-    const QSizeF  plotAreaSize = plotArea->size();
-    if ( label->position().y() + label->size().height() / 2.0
-         < shape->size().height() / 2.0 )
-    {
-        const qreal verticalSpaceRemaining = plotArea->position().y();
-        const qreal spaceToExpand          = ( label->position().y() + label->size().height() ) - verticalSpaceRemaining;
+    if (label->isVisible() == doShow)
+        return;
 
-        if ( spaceToExpand > 0.0 ) {
-            plotArea->setSize( QSizeF( plotAreaSize.width(),
-                                       plotAreaSize.height() - spaceToExpand ) );
-            plotArea->setPosition( QPointF( plotArea->position().x(),
-                                            plotArea->position().y() + spaceToExpand ) );
-        }
+    label->setVisible( doShow );
+
+    // The bounds of the space that is left for the plotarea.
+    // Start with the full height of the shape.
+    qreal yTop = 0;
+    qreal yBottom = shape->size().height();
+
+    // Start calculate the available space for the plotarea.  First
+    // remove the space for the title/subtitle/footer
+    if (title->isVisible()) {
+        yTop = title->position().y() + title->size().height();
     }
-    else {
-        const qreal verticalSpaceRemaining = label->position().y() - plotArea->position().y() - plotArea->size().height();
-        qreal spaceToExpand = ( shape->size().height() - label->position().y() - label->size().height() ) - verticalSpaceRemaining;
-
-        if ( spaceToExpand < 0.0 )
-            spaceToExpand = 0.0;
-
-        foreach ( Axis *axis, plotArea->axes() ) {
-            if ( axis->position() != BottomAxisPosition )
-                continue;
-
-            qreal _spaceToExpand = ( label->size().height()
-                                      - ( shape->size().height()
-                                          - axis->title()->position().y()
-                                          - axis->title()->size().height() ) );
-            if ( _spaceToExpand > 0.0 ) {
-                axis->title()->setPosition( axis->title()->position()
-                                            - QPointF( 0.0, _spaceToExpand ) );
-                spaceToExpand += _spaceToExpand;
-            }
-        }
-
-        if ( spaceToExpand > 0.0 ) {
-            plotArea->setSize( QSizeF( plotAreaSize.width(),
-				       plotAreaSize.height() - spaceToExpand ) );
-        }
+    if (subTitle->isVisible()) {
+        // Here we assume that the subtitle is below the title.
+        yTop = subTitle->position().y() + subTitle->size().height();
+    }
+    if (footer->isVisible()) {
+        yBottom = footer->position().y();
     }
 
-    label->setVisible( true );
+    // Then remove the space that is taken by the axis titles.
+    foreach ( Axis *axis, plotArea->axes() ) {
+        if ( axis->position() != BottomAxisPosition )
+            continue;
+
+        // Make room for the Axis titles.
+        // FIXME: They should only be allocated space if they are visible.
+        yBottom -= axis->title()->size().height();
+        axis->title()->setPosition( QPointF(axis->title()->position().x(), yBottom) );
+    }
+
+    // Check if there is any room left for the plotarea.
+    // Maybe we should use a minimal size?
+    if (yTop < yBottom) {
+        plotArea->setPosition(QPointF(plotArea->position().x(), yTop));
+        plotArea->setSize(QSizeF(plotArea->size().width(), yBottom - yTop));
+    }
 }
 
 
@@ -368,10 +366,11 @@ void ChartShape::Private::showLabel( KoShape *label )
 // ================================================================
 
 
-ChartShape::ChartShape()
+ChartShape::ChartShape(KoResourceManager *resourceManager)
     : KoFrameShape( KoXmlNS::draw, "object" )
     , d ( new Private( this ) )
 {
+    d->resourceManager = resourceManager;
     setShapeId( ChartShapeId );
 
     // Instantiated all children first
@@ -395,14 +394,11 @@ ChartShape::ChartShape()
     setClipping( d->legend, true );
 
     // A few simple defaults (chart type and subtype in this case)
-    d->plotArea->setChartType( BarChartType );
-    d->plotArea->setChartSubType( NormalChartSubtype );
+    setChartType( BarChartType );
+    setChartSubType( NormalChartSubtype );
 
     // Create the Title, which is a standard TextShape.
-    // We use a empty dataCenterMap here for the creation and then in
-    // init() we init the shapes with the correct data.
-    QMap<QString, KoDataCenter *> dataCenterMap;
-    d->title = KoShapeRegistry::instance()->value( TextShapeId )->createDefaultShapeAndInit( dataCenterMap );
+    d->title = KoShapeRegistry::instance()->value(TextShapeId)->createDefaultShape(resourceManager);
     if ( !d->title ) {
         d->title = new TextLabelDummy;
         KMessageBox::error( 0, i18n("The plugin needed for displaying text labels in a chart is not available."), i18n("Plugin Missing") );
@@ -414,12 +410,17 @@ ChartShape::ChartShape()
         d->title->setUserData( dataDummy );
     }
 
+    // Start with a reasonable default size that we can base all following relative
+    // positions of chart elements on.
+    KoShape::setSize( QSizeF( CM_TO_POINT( 8 ), CM_TO_POINT( 5 ) ) );
+
     // Add the title to the shape
     addChild( d->title );
     QFont font = titleData()->document()->defaultFont();
     font.setPointSizeF( 12.0 );
     titleData()->document()->setDefaultFont( font );
     titleData()->document()->setHtml( "<div align=\"center\">" + i18n( "Title" ) + "</font></div>" );
+    // Position the title center at the very top.
     d->title->setSize( QSizeF( CM_TO_POINT( 5 ), CM_TO_POINT( 0.7 ) ) );
     d->title->setPosition( QPointF( size().width() / 2.0 - d->title->size().width() / 2.0, 0.0 ) );
     d->title->setVisible( false );
@@ -427,7 +428,7 @@ ChartShape::ChartShape()
     setClipping( d->title, true );
 
     // Create the Subtitle and add it to the shape.
-    d->subTitle = KoShapeRegistry::instance()->value( TextShapeId )->createDefaultShapeAndInit( dataCenterMap );
+    d->subTitle = KoShapeRegistry::instance()->value(TextShapeId)->createDefaultShape(resourceManager);
     if ( !d->subTitle ) {
         d->subTitle = new TextLabelDummy;
     }
@@ -440,6 +441,8 @@ ChartShape::ChartShape()
     font.setPointSizeF( 10.0 );
     subTitleData()->document()->setDefaultFont( font );
     subTitleData()->document()->setHtml( "<div align=\"center\">" + i18n( "Subtitle" ) + "</div>" );
+
+    // Position it in the center, just below the title.
     d->subTitle->setSize( QSizeF( CM_TO_POINT( 5 ), CM_TO_POINT( 0.6 ) ) );
     d->subTitle->setPosition( QPointF( size().width() / 2.0 - d->title->size().width() / 2.0,
                                        d->title->size().height() ) );
@@ -448,7 +451,7 @@ ChartShape::ChartShape()
     setClipping( d->subTitle, true );
 
     // Create the Footer and add it to the shape.
-    d->footer = KoShapeRegistry::instance()->value( TextShapeId )->createDefaultShapeAndInit( dataCenterMap );
+    d->footer = KoShapeRegistry::instance()->value(TextShapeId)->createDefaultShape(resourceManager);
     if ( !d->footer ) {
         d->footer = new TextLabelDummy;
     }
@@ -461,8 +464,11 @@ ChartShape::ChartShape()
     font.setPointSizeF( 10.0 );
     footerData()->document()->setDefaultFont( font );
     footerData()->document()->setHtml( "<div align=\"center\">" + i18n( "Footer" ) + "</div>" );
+
+    // Position the footer in the center, at the bottom.
     d->footer->setSize( QSizeF( CM_TO_POINT( 5 ), CM_TO_POINT( 0.6 ) ) );
-    d->footer->setPosition( QPointF( size().width() / 2.0 - d->footer->size().width() / 2.0, size().height() - d->footer->size().height() ) );
+    d->footer->setPosition( QPointF( size().width() / 2.0 - d->footer->size().width() / 2.0,
+                                     size().height() - d->footer->size().height() ) );
     d->footer->setVisible( false );
     d->footer->setZIndex( 4 );
     setClipping( d->footer, true );
@@ -473,9 +479,6 @@ ChartShape::ChartShape()
     KoLineBorder *border = new KoLineBorder( 0, Qt::black );
     setBorder( border );
 
-    // Default size of the chart.
-    KoShape::setSize( QSizeF( CM_TO_POINT( 8 ), CM_TO_POINT( 5 ) ) );
-
     requestRepaint();
 }
 
@@ -557,19 +560,19 @@ PlotArea *ChartShape::plotArea() const
 }
 
 
-void ChartShape::showTitle()
+void ChartShape::showTitle(bool doShow)
 {
-    d->showLabel( d->title );
+    d->showLabel( d->title, doShow );
 }
 
-void ChartShape::showSubTitle()
+void ChartShape::showSubTitle(bool doShow)
 {
-    d->showLabel( d->subTitle );
+    d->showLabel( d->subTitle, doShow );
 }
 
-void ChartShape::showFooter()
+void ChartShape::showFooter(bool doShow)
 {
-    d->showLabel( d->footer );
+    d->showLabel( d->footer, doShow );
 }
 
 void ChartShape::setModel( QAbstractItemModel *model,
@@ -671,6 +674,16 @@ static QPointF scalePointCenterBottom( QPointF center, qreal factorX, qreal fact
     return center;
 }
 
+QPointF ChartShape::Private::relativePosition( KoShape *shape )
+{
+    return shape->absolutePosition() - this->shape->absolutePosition( KoFlake::TopLeftCorner );
+}
+
+void ChartShape::Private::setRelativePosition( KoShape *shape, const QPointF &relPos )
+{
+    shape->setAbsolutePosition( relPos + this->shape->absolutePosition( KoFlake::TopLeftCorner ) );
+}
+
 void ChartShape::setSize( const QSizeF &newSize )
 {
     Q_ASSERT( d->plotArea );
@@ -683,16 +696,20 @@ void ChartShape::setSize( const QSizeF &newSize )
         KoShape *title = axis->title();
         switch( axis->position() ) {
         case TopAxisPosition:
-            title->setAbsolutePosition( scalePointCenterTop( title->absolutePosition(), factorX, factorY, title->boundingRect().size() ) );
+            d->setRelativePosition( title,
+                                    scalePointCenterTop( d->relativePosition( title ), factorX, factorY, title->boundingRect().size() ) );
             break;
         case BottomAxisPosition:
-            title->setAbsolutePosition( scalePointCenterBottom( title->absolutePosition(), factorX, factorY, title->boundingRect().size() ) );
+            d->setRelativePosition( title,
+                                    scalePointCenterBottom( d->relativePosition( title ), factorX, factorY, title->boundingRect().size() ) );
             break;
         case LeftAxisPosition:
-            title->setAbsolutePosition( scalePointCenterLeft( title->absolutePosition(), factorX, factorY, title->boundingRect().size() ) );
+            d->setRelativePosition( title,
+                                    scalePointCenterLeft( d->relativePosition( title ), factorX, factorY, title->boundingRect().size() ) );
             break;
         case RightAxisPosition:
-            title->setAbsolutePosition( scalePointCenterRight( title->absolutePosition(), factorX, factorY, title->boundingRect().size() ) );
+            d->setRelativePosition( title,
+                                    scalePointCenterRight( d->relativePosition( title ), factorX, factorY, title->boundingRect().size() ) );
             break;
         }
     }
@@ -700,16 +717,20 @@ void ChartShape::setSize( const QSizeF &newSize )
     // Reposition the Legend within the shape.
     switch ( d->legend->legendPosition() ) {
     case TopLegendPosition:
-        d->legend->setAbsolutePosition( scalePointCenterTop( d->legend->absolutePosition(), factorX, factorY, d->legend->boundingRect().size() ) );
+        d->setRelativePosition( d->legend,
+                                scalePointCenterTop( d->relativePosition( d->legend ), factorX, factorY, d->legend->boundingRect().size() ) );
         break;
     case BottomLegendPosition:
-        d->legend->setAbsolutePosition( scalePointCenterBottom( d->legend->absolutePosition(), factorX, factorY, d->legend->boundingRect().size() ) );
+        d->setRelativePosition( d->legend,
+                                scalePointCenterBottom( d->relativePosition( d->legend ), factorX, factorY, d->legend->boundingRect().size() ) );
         break;
     case StartLegendPosition:
-        d->legend->setAbsolutePosition( scalePointCenterLeft( d->legend->absolutePosition(), factorX, factorY, d->legend->boundingRect().size() ) );
+        d->setRelativePosition( d->legend,
+                                scalePointCenterLeft( d->relativePosition( d->legend ), factorX, factorY, d->legend->boundingRect().size() ) );
         break;
     case EndLegendPosition:
-        d->legend->setAbsolutePosition( scalePointCenterRight( d->legend->absolutePosition(), factorX, factorY, d->legend->boundingRect().size() ) );
+        d->setRelativePosition( d->legend,
+                                scalePointCenterRight( d->relativePosition( d->legend ), factorX, factorY, d->legend->boundingRect().size() ) );
         break;
     case TopStartLegendPosition:
     case BottomStartLegendPosition:
@@ -720,9 +741,12 @@ void ChartShape::setSize( const QSizeF &newSize )
     }
 
     // Reposition the Title, Subtitle and Footer within the shape.
-    d->title->setAbsolutePosition( scalePointCenterTop( d->title->absolutePosition(), factorX, factorY, d->title->boundingRect().size() ) );
-    d->subTitle->setAbsolutePosition( scalePointCenterTop( d->subTitle->absolutePosition(), factorX, factorY, d->subTitle->boundingRect().size() ) );
-    d->footer->setAbsolutePosition( scalePointCenterBottom( d->footer->absolutePosition(), factorX, factorY, d->footer->boundingRect().size() ) );
+    d->setRelativePosition( d->title,
+                            scalePointCenterTop( d->relativePosition( d->title ), factorX, factorY, d->title->boundingRect().size() ) );
+    d->setRelativePosition( d->subTitle,
+                            scalePointCenterTop( d->relativePosition( d->subTitle ), factorX, factorY, d->subTitle->boundingRect().size() ) );
+    d->setRelativePosition( d->footer,
+                            scalePointCenterBottom( d->relativePosition( d->footer ), factorX, factorY, d->footer->boundingRect().size() ) );
 
     // Finally, resize the plotarea.
     const QSizeF plotAreaSize = d->plotArea->size();
@@ -733,6 +757,51 @@ void ChartShape::setSize( const QSizeF &newSize )
     KoShape::setSize( newSize );
 }
 
+void ChartShape::setLegendSize( const QSizeF &size )
+{
+    QPointF newPos = d->legend->position();
+    QSizeF oldSize = d->legend->size();
+    switch ( d->legend->legendPosition() ) {
+    case TopLegendPosition:
+        // Move it to the left as much as the width changed
+        newPos.rx() -= (size.width() - oldSize.width()) / 2.0;
+        // No adjustment of y position needed
+        break;
+    case BottomLegendPosition:
+        // Move it up as much as the height changed
+        newPos.ry() -= (size.height() - oldSize.height());
+        // Move it to the left as much as the width changed
+        newPos.rx() -= (size.width() - oldSize.width()) / 2.0;
+        break;
+    case StartLegendPosition:
+        // No adjustment of x position needed
+        newPos.ry() -= (size.height() - oldSize.height()) / 2.0;
+        break;
+    case EndLegendPosition:
+        newPos.ry() -= (size.height() - oldSize.height()) / 2.0;
+        // Move it to the left as much as the width changed
+        newPos.rx() -= (size.width() - oldSize.width());
+        break;
+    // FIXME: The following positions are not handled
+    // The new position is simply set as is
+    case TopStartLegendPosition:
+    case BottomStartLegendPosition:
+    case TopEndLegendPosition:
+    case BottomEndLegendPosition:
+
+    // If the legend is floating, the user has changed the position manually.
+    // There's no way to determine where the user wants it to be after a resize.
+    // (more to the left? more to the right?)
+    case FloatingLegendPosition:
+        break;
+    }
+
+    // TODO: Adjust plot area's size and position if necessary
+
+    d->legend->setPosition( newPos );
+    d->legend->setSize( size );
+}
+
 void ChartShape::updateChildrenPositions()
 {
     Q_ASSERT( d->plotArea );
@@ -745,8 +814,7 @@ void ChartShape::updateChildrenPositions()
         title->setPosition( titlePosition );
     }
 
-    const qreal legendXOffset = 10.0;
-    d->legend->setPosition( QPointF( size().width() + legendXOffset,
+    d->legend->setPosition( QPointF( size().width() - d->legend->size().width(),
                                      size().height() / 2.0 - d->legend->size().height() / 2.0 ) );
 }
 
@@ -797,6 +865,7 @@ void ChartShape::setChartType( ChartType type )
 {
     Q_ASSERT( d->plotArea );
     d->plotArea->setChartType( type );
+    emit chartTypeChanged( type );
 }
 
 void ChartShape::setChartSubType( ChartSubtype subType )
@@ -995,6 +1064,13 @@ bool ChartShape::loadEmbeddedDocument( KoStore *store,
 bool ChartShape::loadOdf( const KoXmlElement &element,
                           KoShapeLoadingContext &context )
 {
+    // Invalidate all data sets before we new load data sets from ODF
+    // and *before* we delete existing axes.
+    proxyModel()->invalidateDataSets();
+
+    // When loading from ODF, all data sets are added explicitly.
+    proxyModel()->setAutomaticDataSetCreation( false );
+
     // Load common attributes of (frame) shapes.  If you change here,
     // don't forget to also change in saveOdf().
     loadOdfAttributes( element, context, OdfMandatories | OdfGeometry | OdfAdditionalAttributes );
@@ -1029,6 +1105,14 @@ bool ChartShape::loadOdfEmbedded( const KoXmlElement &chartElement,
     loadOdfAttributes( chartElement, context,
                        OdfAdditionalAttributes | OdfMandatories | OdfCommonChildElements | OdfStyle );
 
+#ifndef NWORKAROUND_ODF_BUGS
+    if ( !background() ) {
+        const QColor color = KoOdfWorkaround::fixMissingFillColor( chartElement, context );
+        if(color.isValid()) // invalid color means do not set KoColorBackground but be transparent instead
+            setBackground( new KoColorBackground( color ) );
+    }
+#endif
+
     // Check if we're loading an embedded document
     if ( !chartElement.hasAttributeNS( KoXmlNS::chart, "class" ) ) {
         kDebug(35001) << "Error: Embedded document has no chart:class attribute.";
@@ -1047,7 +1131,7 @@ bool ChartShape::loadOdfEmbedded( const KoXmlElement &chartElement,
         if ( chartClass == ODF_CHARTTYPES[ (ChartType)type ] ) {
             //kDebug(35001) <<"found chart of type" << chartClass;
 
-            d->plotArea->setChartType( (ChartType)type );
+            setChartType( (ChartType)type );
             knownType = true;
             break;
         }
@@ -1061,40 +1145,7 @@ bool ChartShape::loadOdfEmbedded( const KoXmlElement &chartElement,
         return false;
     }
 
-    // 2. Load the title.
-    KoXmlElement titleElem = KoXml::namedItemNS( chartElement,
-                                                 KoXmlNS::chart, "title" );
-    if ( !titleElem.isNull() ) {
-        if ( !loadOdfLabel( d->title, titleElem, context) )
-            return false;
-    }
-
-    // 3. Load the subtitle.
-    KoXmlElement subTitleElem = KoXml::namedItemNS( chartElement,
-                                                    KoXmlNS::chart, "subtitle" );
-    if ( !subTitleElem.isNull() ) {
-        if ( !loadOdfLabel( d->subTitle, subTitleElem, context) )
-            return false;
-    }
-
-    // 4. Load the footer.
-    KoXmlElement footerElem = KoXml::namedItemNS( chartElement,
-                                                  KoXmlNS::chart, "footer" );
-    if ( !footerElem.isNull() ) {
-        if ( !loadOdfLabel( d->footer, footerElem, context ) )
-            return false;
-    }
-
-    // 5. Load the legend.
-    KoXmlElement legendElem = KoXml::namedItemNS( chartElement, KoXmlNS::chart,
-                          "legend" );
-    if ( !legendElem.isNull() ) {
-        if ( !d->legend->loadOdf( legendElem, context ) )
-            return false;
-    }
-    d->legend->update();
-
-    // 6. Load the data
+    // 2. Load the data
     KoXmlElement  dataElem = KoXml::namedItemNS( chartElement,
                                                  KoXmlNS::table, "table" );
     if ( !dataElem.isNull() ) {
@@ -1102,7 +1153,7 @@ bool ChartShape::loadOdfEmbedded( const KoXmlElement &chartElement,
             return false;
     }
 
-    // 7. Load the plot area (this is where the meat is!).
+    // 3. Load the plot area (this is where the meat is!).
     KoXmlElement  plotareaElem = KoXml::namedItemNS( chartElement,
                                                      KoXmlNS::chart, "plot-area" );
     if ( !plotareaElem.isNull() ) {
@@ -1110,6 +1161,42 @@ bool ChartShape::loadOdfEmbedded( const KoXmlElement &chartElement,
             return false;
     }
 
+    // 4. Load the title.
+    KoXmlElement titleElem = KoXml::namedItemNS( chartElement,
+                                                 KoXmlNS::chart, "title" );
+    if ( !titleElem.isNull() ) {
+        if ( !loadOdfLabel( d->title, titleElem, context) )
+            return false;
+        d->showLabel(d->title, true);
+    }
+
+    // 5. Load the subtitle.
+    KoXmlElement subTitleElem = KoXml::namedItemNS( chartElement,
+                                                    KoXmlNS::chart, "subtitle" );
+    if ( !subTitleElem.isNull() ) {
+        if ( !loadOdfLabel( d->subTitle, subTitleElem, context) )
+            return false;
+        d->showLabel(d->subTitle, true);
+    }
+
+    // 6. Load the footer.
+    KoXmlElement footerElem = KoXml::namedItemNS( chartElement,
+                                                  KoXmlNS::chart, "footer" );
+    if ( !footerElem.isNull() ) {
+        if ( !loadOdfLabel( d->footer, footerElem, context ) )
+            return false;
+        d->showLabel(d->footer, true);
+    }
+
+    // 7. Load the legend.
+    KoXmlElement legendElem = KoXml::namedItemNS( chartElement, KoXmlNS::chart,
+                          "legend" );
+    if ( !legendElem.isNull() ) {
+        if ( !d->legend->loadOdf( legendElem, context ) )
+            return false;
+    }
+    d->legend->update();
+
     requestRepaint();
 
     styleStack.restore();
@@ -1176,7 +1263,7 @@ void ChartShape::saveOdf( KoShapeSavingContext & context ) const
     saveOdfAttributes( context, OdfAllAttributes ^ OdfMandatories );
 
     KoGenStyle style;
-    style = KoGenStyle( KoGenStyle::StyleGraphicAuto, "chart" );
+    style = KoGenStyle( KoGenStyle::GraphicAutoStyle, "chart" );
     bodyWriter.addAttribute( "chart:style-name", saveStyle( style, context ) );
 
     // 1. Write the chart type.
@@ -1300,14 +1387,6 @@ void ChartShape::saveOdfData( KoXmlWriter &bodyWriter, KoGenStyles &mainStyles )
     bodyWriter.endElement(); // table:table
 }
 
-void ChartShape::init( const QMap<QString, KoDataCenter *> & dataCenterMap )
-{
-    d->dataCenterMap = dataCenterMap;
-    d->title->init( dataCenterMap );
-    d->subTitle->init( dataCenterMap );
-    d->footer->init( dataCenterMap );
-}
-
 void ChartShape::update() const
 {
     KoShape::update();
@@ -1326,9 +1405,9 @@ void ChartShape::requestRepaint() const
     d->plotArea->requestRepaint();
 }
 
-QMap<QString, KoDataCenter*> ChartShape::dataCenterMap() const
+KoResourceManager *ChartShape::resourceManager() const
 {
-    return d->dataCenterMap;
+    return d->resourceManager;
 }
 
 } // Namespace KChart
