@@ -85,7 +85,7 @@ class Opcode
 public:
 
     enum { Nop = 0, Load, Ref, Cell, Range, Function, Add, Sub, Neg, Mul, Div,
-           Pow, Concat, Not, Equal, Less, Greater, Array
+           Pow, Concat, Intersect, Not, Equal, Less, Greater, Array
          };
 
     unsigned type;
@@ -106,6 +106,8 @@ public:
     QString expression;
     mutable QVector<Opcode> codes;
     mutable QVector<Value> constants;
+
+    Value valueOrElement(const Map* map, int index, FuncExtra &fe, const Value &v) const;
 };
 
 class TokenStack : public QVector<Token>
@@ -146,6 +148,7 @@ Token::Op KSpread::matchOperator(const QString& text)
         case '^': result = Token::Caret; break;
         case ',': result = Token::Comma; break;
         case ';': result = Token::Semicolon; break;
+        case ' ': result = Token::Intersect; break;
         case '(': result = Token::LeftPar; break;
         case ')': result = Token::RightPar; break;
         case '&': result = Token::Ampersand; break;
@@ -192,6 +195,7 @@ static int opPrecedence(Token::Op op)
     case Token::Plus         : prec = 3; break;
     case Token::Minus        : prec = 3; break;
     case Token::Ampersand    : prec = 2; break;
+    case Token::Intersect    : prec = 2; break;
     case Token::Equal        : prec = 1; break;
     case Token::NotEqual     : prec = 1; break;
     case Token::Less         : prec = 1; break;
@@ -568,11 +572,11 @@ Tokens Formula::scan(const QString& expr, const KLocale* locale) const
 
             tokenStart = i;
 
-            // skip any whitespaces
-            if (ch.isSpace()) i++;
+            // don't just skip whitespaces cause they could also be used to intersect two arrays
+            //if (ch.isSpace()) { i++; break; }
 
             // check for number
-            else if (ch.isDigit()) {
+            if (ch.isDigit()) {
                 state = InNumber;
             }
 
@@ -1196,6 +1200,7 @@ void Formula::compile(const Tokens& tokens) const
                                                 case Token::Slash:        d->codes.append(Opcode::Div); break;
                                                 case Token::Caret:        d->codes.append(Opcode::Pow); break;
                                                 case Token::Ampersand:    d->codes.append(Opcode::Concat); break;
+                                                case Token::Intersect:    d->codes.append(Opcode::Intersect); break;
 
                                                     // simple value comparisons
                                                 case Token::Equal:        d->codes.append(Opcode::Equal); break;
@@ -1316,8 +1321,26 @@ Value Formula::eval(CellIndirection cellIndirections) const
     return evalRecursive(cellIndirections, values);
 }
 
+// We need to unroll arrays. Do use the same logic to unroll like OpenOffice.org and Excel are using.
+Value Formula::Private::valueOrElement(const Map* map, int index, FuncExtra &fe, const Value &v) const
+{
+    if(v.isArray()) {
+        if(v.count() == 1) // if there is only one item, use that one
+            return v.element(0);
+        QString c = constants[index].asString();
+        const Region region(c, map, sheet);
+        if(region.isValid()) {
+            const QPoint position = region.firstRange().topLeft();
+            const int idx = fe.myrow - position.y(); // do we need to do the same for columns?
+            if(idx >= 0 && idx < int(v.count()))
+                return v.element(idx); // within the range returns the selected element
+        }
+    }
+    return v;
+}
+
 // On OO.org Calc and MS Excel operations done with +, -, * and / do fail if one of the values is
-// non-numeric. This differs from formulas like SUM which just ignore non numeric values.
+// non-numeric. This differs from formulas like SUM which just ignores non numeric values.
 Value numericOrError(const ValueConverter* converter, const Value &v)
 {
     switch (v.type()) {
@@ -1338,7 +1361,7 @@ Value numericOrError(const ValueConverter* converter, const Value &v)
         } break;
         case Value::Array:
         case Value::CellRange:
-            break;
+            return v;
     }
     return Value::errorVALUE();
 }
@@ -1393,7 +1416,7 @@ Value Formula::evalRecursive(CellIndirection cellIndirections, QHash<Cell, Value
             // unary operation
         case Opcode::Neg:
             entry.reset();
-            entry.val = stack.pop().val;
+            entry.val = d->valueOrElement(map, index, fe, stack.pop().val);
             if (!entry.val.isError()) // do nothing if we got an error
                 entry.val = calc->mul(entry.val, -1);
             stack.push(entry);
@@ -1403,8 +1426,8 @@ Value Formula::evalRecursive(CellIndirection cellIndirections, QHash<Cell, Value
             // push the result to stack
         case Opcode::Add:
             entry.reset();
-            val2 = numericOrError(converter, stack.pop().val);
-            val1 = numericOrError(converter, stack.pop().val);
+            val2 = numericOrError(converter, d->valueOrElement(map, index, fe, stack.pop().val));
+            val1 = numericOrError(converter, d->valueOrElement(map, index, fe, stack.pop().val));
             val2 = calc->add(val1, val2);
             entry.reset();
             entry.val = val2;
@@ -1412,8 +1435,8 @@ Value Formula::evalRecursive(CellIndirection cellIndirections, QHash<Cell, Value
             break;
 
         case Opcode::Sub:
-            val2 = numericOrError(converter, stack.pop().val);
-            val1 = numericOrError(converter, stack.pop().val);
+            val2 = numericOrError(converter, d->valueOrElement(map, index, fe, stack.pop().val));
+            val1 = numericOrError(converter, d->valueOrElement(map, index, fe, stack.pop().val));
             val2 = calc->sub(val1, val2);
             entry.reset();
             entry.val = val2;
@@ -1421,8 +1444,8 @@ Value Formula::evalRecursive(CellIndirection cellIndirections, QHash<Cell, Value
             break;
 
         case Opcode::Mul:
-            val2 = numericOrError(converter, stack.pop().val);
-            val1 = numericOrError(converter, stack.pop().val);
+            val2 = numericOrError(converter, d->valueOrElement(map, index, fe, stack.pop().val));
+            val1 = numericOrError(converter, d->valueOrElement(map, index, fe, stack.pop().val));
             val2 = calc->mul(val1, val2);
             entry.reset();
             entry.val = val2;
@@ -1430,8 +1453,8 @@ Value Formula::evalRecursive(CellIndirection cellIndirections, QHash<Cell, Value
             break;
 
         case Opcode::Div:
-            val2 = numericOrError(converter, stack.pop().val);
-            val1 = numericOrError(converter, stack.pop().val);
+            val2 = numericOrError(converter, d->valueOrElement(map, index, fe, stack.pop().val));
+            val1 = numericOrError(converter, d->valueOrElement(map, index, fe, stack.pop().val));
             val2 = calc->div(val1, val2);
             entry.reset();
             entry.val = val2;
@@ -1439,8 +1462,8 @@ Value Formula::evalRecursive(CellIndirection cellIndirections, QHash<Cell, Value
             break;
 
         case Opcode::Pow:
-            val2 = stack.pop().val;
-            val1 = stack.pop().val;
+            val2 = numericOrError(converter, d->valueOrElement(map, index, fe, stack.pop().val));
+            val1 = numericOrError(converter, d->valueOrElement(map, index, fe, stack.pop().val));
             val2 = calc->pow(val1, val2);
             entry.reset();
             entry.val = val2;
@@ -1460,9 +1483,37 @@ Value Formula::evalRecursive(CellIndirection cellIndirections, QHash<Cell, Value
             stack.push(entry);
             break;
 
+           // array intersection
+        case Opcode::Intersect: {
+            val1 = stack.pop().val;
+            val2 = stack.pop().val;
+            Region r1(d->constants[index].asString(), map, d->sheet);
+            Region r2(d->constants[index+1].asString(), map, d->sheet);
+            if(!r1.isValid() || !r2.isValid()) {
+                val1 = Value::errorNULL();
+            } else {
+                Region r = r1.intersected(r2);
+                QRect rect = r.boundingRect();
+                Cell cell;
+                if(rect.top() == rect.bottom())
+                    cell = Cell(r.firstSheet(), fe.mycol, rect.top());
+                else if(rect.left() == rect.right())
+                    cell = Cell(r.firstSheet(), rect.left(), fe.mycol);
+                if(cell.isNull())
+                    val1 = Value::errorNULL();
+                else if(cell.isEmpty())
+                    val1 = Value::errorNULL();
+                else
+                    val1 = cell.value();
+            }
+            entry.reset();
+            entry.val = val1;
+            stack.push(entry);
+        } break;
+
             // logical not
         case Opcode::Not:
-            val1 = converter->asBoolean(stack.pop().val);
+            val1 = converter->asBoolean(d->valueOrElement(map, index, fe, stack.pop().val));
             if (val1.isError())
                 val1 = Value::errorVALUE();
             else
@@ -1474,10 +1525,12 @@ Value Formula::evalRecursive(CellIndirection cellIndirections, QHash<Cell, Value
 
             // comparison
         case Opcode::Equal:
-            val1 = stack.pop().val;
-            val2 = stack.pop().val;
-            if (!val1.allowComparison(val2))
-                val1 = Value::errorNA();
+            val1 = d->valueOrElement(map, index, fe, stack.pop().val);
+            val2 = d->valueOrElement(map, index, fe, stack.pop().val);
+            if (val1.isError())
+                ;
+            else if (val2.isError())
+                val1 = val2;
             else if (val2.compare(val1) == 0)
                 val1 = Value(true);
             else
@@ -1489,10 +1542,12 @@ Value Formula::evalRecursive(CellIndirection cellIndirections, QHash<Cell, Value
 
             // less than
         case Opcode::Less:
-            val1 = stack.pop().val;
-            val2 = stack.pop().val;
-            if (!val1.allowComparison(val2))
-                val1 = Value::errorNA();
+            val1 = d->valueOrElement(map, index, fe, stack.pop().val);
+            val2 = d->valueOrElement(map, index, fe, stack.pop().val);
+            if (val1.isError())
+                ;
+            else if (val2.isError())
+                val1 = val2;
             else if (val2.compare(val1) < 0)
                 val1 = Value(true);
             else
@@ -1504,10 +1559,12 @@ Value Formula::evalRecursive(CellIndirection cellIndirections, QHash<Cell, Value
 
             // greater than
         case Opcode::Greater: {
-            val1 = stack.pop().val;
-            val2 = stack.pop().val;
-            if (!val1.allowComparison(val2))
-                val1 = Value::errorNA();
+            val1 = d->valueOrElement(map, index, fe, stack.pop().val);
+            val2 = d->valueOrElement(map, index, fe, stack.pop().val);
+            if (val1.isError())
+                ;
+            else if (val2.isError())
+                val1 = val2;
             else if (val2.compare(val1) > 0)
                 val1 = Value(true);
             else
@@ -1548,8 +1605,9 @@ Value Formula::evalRecursive(CellIndirection cellIndirections, QHash<Cell, Value
                 // store the reference, so we can use it within functions
                 entry.col1 = entry.col2 = position.x();
                 entry.row1 = entry.row2 = position.y();
+            } else {
+                kWarning() << "Unhandled non singular region in Opcode::Cell with rects=" << region.rects();
             }
-
             entry.val = val1;
             stack.push(entry);
         }
@@ -1571,7 +1629,7 @@ Value Formula::evalRecursive(CellIndirection cellIndirections, QHash<Cell, Value
                 entry.row2 = region.firstRange().bottom();
             }
 
-            entry.val = val1;
+            entry.val = val1; // any array is valid here
             stack.push(entry);
         }
         break;
@@ -1608,7 +1666,7 @@ Value Formula::evalRecursive(CellIndirection cellIndirections, QHash<Cell, Value
             // function name as string value
             val1 = converter->asString(stack.pop().val);
             if (val1.isError())
-                return Value::errorVALUE();
+                return val1;
             function = FunctionRepository::self()->function(val1.asString());
             if (!function)
                 return Value::errorNAME(); // no such function
@@ -1701,21 +1759,22 @@ QString Formula::dump() const
     for (int i = 0; i < d->codes.count(); i++) {
         QString ctext;
         switch (d->codes[i].type) {
-        case Opcode::Load:     ctext = QString("Load #%1").arg(d->codes[i].index); break;
-        case Opcode::Ref:      ctext = QString("Ref #%1").arg(d->codes[i].index); break;
-        case Opcode::Function: ctext = QString("Function (%1)").arg(d->codes[i].index); break;
-        case Opcode::Add:      ctext = "Add"; break;
-        case Opcode::Sub:      ctext = "Sub"; break;
-        case Opcode::Mul:      ctext = "Mul"; break;
-        case Opcode::Div:      ctext = "Div"; break;
-        case Opcode::Neg:      ctext = "Neg"; break;
-        case Opcode::Concat:   ctext = "Concat"; break;
-        case Opcode::Pow:      ctext = "Pow"; break;
-        case Opcode::Equal:    ctext = "Equal"; break;
-        case Opcode::Not:      ctext = "Not"; break;
-        case Opcode::Less:     ctext = "Less"; break;
-        case Opcode::Greater:  ctext = "Greater"; break;
-        case Opcode::Array:    ctext = QString("Array (%1x%2)").arg(d->constants[d->codes[i].index].asInteger()).arg(d->constants[d->codes[i].index+1].asInteger()); break;
+        case Opcode::Load:      ctext = QString("Load #%1").arg(d->codes[i].index); break;
+        case Opcode::Ref:       ctext = QString("Ref #%1").arg(d->codes[i].index); break;
+        case Opcode::Function:  ctext = QString("Function (%1)").arg(d->codes[i].index); break;
+        case Opcode::Add:       ctext = "Add"; break;
+        case Opcode::Sub:       ctext = "Sub"; break;
+        case Opcode::Mul:       ctext = "Mul"; break;
+        case Opcode::Div:       ctext = "Div"; break;
+        case Opcode::Neg:       ctext = "Neg"; break;
+        case Opcode::Concat:    ctext = "Concat"; break;
+        case Opcode::Pow:       ctext = "Pow"; break;
+        case Opcode::Intersect: ctext = "Intersect"; break;
+        case Opcode::Equal:     ctext = "Equal"; break;
+        case Opcode::Not:       ctext = "Not"; break;
+        case Opcode::Less:      ctext = "Less"; break;
+        case Opcode::Greater:   ctext = "Greater"; break;
+        case Opcode::Array:     ctext = QString("Array (%1x%2)").arg(d->constants[d->codes[i].index].asInteger()).arg(d->constants[d->codes[i].index+1].asInteger()); break;
         default: ctext = "Unknown"; break;
         }
         result.append("   ").append(ctext).append("\n");
