@@ -53,7 +53,7 @@
 #include <ktemporaryfile.h>
 
 #include <KoApplication.h>
-#include <KoDataCenter.h>
+#include <KoDataCenterBase.h>
 #include <KoDocumentInfo.h>
 #include <KoMainWindow.h>
 #include <KoOasisSettings.h>
@@ -61,8 +61,8 @@
 #include <KoOdfStylesReader.h>
 #include <KoOdfReadStore.h>
 #include <KoOdfWriteStore.h>
-#include <KoShapeConfigFactory.h>
-#include <KoShapeFactory.h>
+#include <KoShapeConfigFactoryBase.h>
+#include <KoShapeFactoryBase.h>
 #include <KoShapeManager.h>
 #include <KoShapeRegistry.h>
 #include <KoStoreDevice.h>
@@ -71,7 +71,6 @@
 #include <KoXmlWriter.h>
 #include <KoZoomHandler.h>
 #include <KoShapeSavingContext.h>
-#include <KoUndoStack.h>
 
 #include "BindingManager.h"
 #include "CalculationSettings.h"
@@ -93,6 +92,7 @@
 #include "StyleManager.h"
 #include "Util.h"
 #include "View.h"
+#include "SheetAccessModel.h"
 
 // chart shape
 #include "kchart/shape/ChartShape.h"
@@ -111,8 +111,6 @@ class Doc::Private
 {
 public:
     Map *map;
-    QMap<QString, KoDataCenter *>  dataCenterMap;
-
     static QList<Doc*> s_docs;
     static int s_docId;
 
@@ -124,9 +122,10 @@ public:
 
     // document properties
     int syntaxVersion;
-bool configLoadFromFile       : 1;
+    bool configLoadFromFile : 1;
     QStringList spellListIgnoreAll;
     SavedDocParts savedDocParts;
+    SheetAccessModel *sheetAccessModel;
 };
 
 /*****************************************************************************
@@ -156,19 +155,14 @@ Doc::Doc(QWidget *parentWidget, QObject* parent, bool singleViewMode)
     // default document properties
     d->syntaxVersion = CURRENT_SYNTAX_VERSION;
 
-    // Init chart shape factory with KSpread's specific configuration panels.
-    QList<KoShapeConfigFactory*> panels = ChartDialog::panels(this);
-    // Ask every shapefactory to populate the dataCenterMap
-    foreach(QString id, KoShapeRegistry::instance()->keys()) {
-        KoShapeFactory *shapeFactory = KoShapeRegistry::instance()->value(id);
-        shapeFactory->populateDataCenterMap(d->dataCenterMap);
-        if (id == ChartShapeId) {
-            shapeFactory->setOptionPanels(panels);
-        }
-    }
+    d->sheetAccessModel = new SheetAccessModel(d->map);
 
-    // Populate the document undoStack in the dataCenterMap. This can be used later by shapes for their undo/redo mechanism.                                               
-    d->dataCenterMap["UndoStack"] = undoStack();
+    // Init chart shape factory with KSpread's specific configuration panels.
+    KoShapeFactoryBase *chartShape = KoShapeRegistry::instance()->value(ChartShapeId);
+    if (chartShape) {
+        QList<KoShapeConfigFactoryBase*> panels = ChartDialog::panels(this);
+        chartShape->setOptionPanels(panels);
+    }
 
     // Load the function modules.
     FunctionModuleRegistry::instance();
@@ -181,8 +175,7 @@ Doc::~Doc()
         saveConfig();
 
     delete d->map;
-    qDeleteAll(d->dataCenterMap);
-
+    delete d->sheetAccessModel;
     delete d;
 }
 
@@ -225,11 +218,6 @@ Map *Doc::map() const
     return d->map;
 }
 
-QMap<QString, KoDataCenter *> Doc::dataCenterMap() const
-{
-    return d->dataCenterMap;
-}
-
 void Doc::saveConfig()
 {
     if (isEmbedded() || !isReadWrite())
@@ -381,7 +369,7 @@ bool Doc::saveOdfHelper(SavingContext & documentContext, SaveFlag saveFlag,
     bodyWriter->endElement(); ////office:body
 
     // Done with writing out the contents to the tempfile, we can now write out the automatic styles
-    mainStyles.saveOdfAutomaticStyles(contentWriter, false);
+    mainStyles.saveOdfStyles(KoGenStyles::DocumentAutomaticStyles, contentWriter);
 
     documentContext.odfStore.closeContentWriter();
 
@@ -724,9 +712,6 @@ bool Doc::completeLoading(KoStore* store)
 
     setModified(false);
     bool ok = map()->completeLoading(store);
-    foreach(KoDataCenter *dataCenter, d->dataCenterMap) {
-        ok = ok && dataCenter->completeLoading(store);
-    }
     return ok;
 }
 
@@ -762,83 +747,15 @@ void Doc::newZoomAndResolution(bool updateViews, bool /*forPrint*/)
 
 void Doc::paintContent(QPainter& painter, const QRect& rect)
 {
-#ifndef KSPREAD_DOC_ZOOM
-    Q_UNUSED(painter);
-    Q_UNUSED(rect);
-#else
-//     kDebug(36001) <<"paintContent() called on" << rect;
-
-//     ElapsedTime et( "Doc::paintContent1" );
-
-    // choose sheet: the first or the active
-    Sheet* sheet = 0;
-    if (!d->activeSheet)
-        sheet = map()->sheet(0);
-    else
-        sheet = d->activeSheet;
-    if (!sheet)
-        return;
-
-    // save current zoom
-    double oldZoom = m_zoom;
-    // set the resolution once
-    setZoom(1.0 / m_zoomedResolutionX);
-
-    // KSpread support zoom, therefore no need to scale with worldMatrix
-    // Save the translation though.
-    QMatrix matrix = painter.matrix();
-    matrix.setMatrix(1, 0, 0, 1, matrix.dx(), matrix.dy());
-
-    // Unscale the rectangle.
-    QRect prect = rect;
-    prect.setWidth((int)(prect.width() * painter.matrix().m11()));
-    prect.setHeight((int)(prect.height() * painter.matrix().m22()));
-
-    // paint the content, now zoom is correctly set
-    kDebug(36001) << "paintContent-------------------------------------";
-    painter.save();
-    painter.setMatrix(matrix);
-    paintContent(painter, prect, sheet, false);
-    painter.restore();
-
-    // restore zoom
-    setZoom(oldZoom);
-#endif // KSPREAD_DOC_ZOOM
+    paintContent(painter, rect, map()->sheet(0), false);
 }
 
 void Doc::paintContent(QPainter& painter, const QRect& rect, Sheet* sheet, bool drawCursor)
 {
-#ifndef KSPREAD_DOC_ZOOM
-    Q_UNUSED(painter);
-    Q_UNUSED(rect);
-    Q_UNUSED(sheet);
-    Q_UNUSED(drawCursor);
-#else
     Q_UNUSED(drawCursor);
 
-    if (isLoading())
-        return;
-    //    ElapsedTime et( "Doc::paintContent2" );
-
-    double xpos;
-    double ypos;
-    int left_col   = sheet->leftColumn(unzoomItX(rect.x()), xpos);
-    int right_col  = sheet->rightColumn(unzoomItX(rect.right()));
-    int top_row    = sheet->topRow(unzoomItY(rect.y()), ypos);
-    int bottom_row = sheet->bottomRow(unzoomItY(rect.bottom()));
-
-    QPen pen;
-    pen.setWidth(1);
-    painter.setPen(pen);
-
-    /* Update the entire visible area. */
-    Region region;
-    region.add(QRect(left_col, top_row,
-                     right_col - left_col + 1,
-                     bottom_row - top_row + 1), sheet);
-
-    paintCellRegions(painter, rect, 0, region);
-#endif // KSPREAD_DOC_ZOOM
+    QPixmap thumbnail = sheet->generateThumbnail(rect.size());
+    painter.drawPixmap(rect, thumbnail);
 }
 
 void Doc::paintUpdates()
@@ -848,41 +765,6 @@ void Doc::paintUpdates()
     }
 }
 
-void Doc::paintCellRegions(QPainter& painter, const QRect &viewRect,
-                           View* view, const Region& region)
-{
-#ifndef KSPREAD_DOC_ZOOM
-    Q_UNUSED(painter);
-    Q_UNUSED(viewRect);
-    Q_UNUSED(view);
-    Q_UNUSED(region);
-#else
-    //
-    // Clip away children
-    //
-    QRegion rgn = painter.clipRegion();
-    if (rgn.isEmpty())
-        rgn = QRegion(QRect(0, 0, viewRect.width(), viewRect.height()));
-
-//   QMatrix matrix;
-//   if ( view ) {
-//     matrix.scale( zoomedResolutionX(),
-//                   zoomedResolutionY() );
-//     matrix.translate( - view->canvasWidget()->xOffset(),
-//                       - view->canvasWidget()->yOffset() );
-//   }
-//   else {
-//     matrix = painter.matrix();
-//   }
-    painter.setClipRegion(rgn);
-
-    Region::ConstIterator endOfList(region.constEnd());
-    for (Region::ConstIterator it = region.constBegin(); it != endOfList; ++it) {
-        paintRegion(painter, viewToDocument(viewRect), view, (*it)->rect(), (*it)->sheet());
-    }
-#endif // KSPREAD_DOC_ZOOM
-}
-
 void Doc::paintRegion(QPainter &painter, const QRectF &viewRegion,
                       View* view, const QRect &cellRegion, const Sheet* sheet)
 {
@@ -978,6 +860,10 @@ void Doc::repaint(const QRectF& rect)
     }
 }
 
+SheetAccessModel *Doc::sheetAccessModel() const
+{
+    return d->sheetAccessModel;
+}
 
 #if 0 // UNDOREDOLIMIT
 int Doc::undoRedoLimit() const
