@@ -1,7 +1,7 @@
 /* This file is part of the KDE project
    Copyright (C) 2006 Casper Boemann Rasmussen <cbr@boemann.dk>
    Copyright (C) 2006-2009 Thomas Zander <zander@kde.org>
-   Copyright (C) 2006-2008 Thorsten Zachmann <zachmann@kde.org>
+   Copyright (C) 2006-2010 Thorsten Zachmann <zachmann@kde.org>
    Copyright (C) 2007-2009 Jan Hambrecht <jaham@gmx.net>
 
    This library is free software; you can redistribute it and/or
@@ -106,13 +106,13 @@ KoShapePrivate::~KoShapePrivate()
     }
     delete userData;
     delete appData;
-    if (border && ! border->removeUser())
+    if (border && !border->deref())
         delete border;
-    if (shadow && ! shadow->removeUser())
+    if (shadow && !shadow->deref())
         delete shadow;
-    if (fill && ! fill->removeUser())
+    if (fill && !fill->deref())
         delete fill;
-    if (filterEffectStack && ! filterEffectStack->removeUser())
+    if (filterEffectStack && !filterEffectStack->deref())
         delete filterEffectStack;
     qDeleteAll(eventActions);
 }
@@ -158,6 +158,20 @@ void KoShapePrivate::removeShapeManager(KoShapeManager *manager)
 {
     shapeManagers.remove(manager);
 }
+// static
+QString KoShapePrivate::getStyleProperty(const char *property, const KoXmlElement &element, KoShapeLoadingContext &context)
+{
+    Q_UNUSED(element);
+    KoStyleStack &styleStack = context.odfLoadingContext().styleStack();
+    QString value;
+
+    if (styleStack.hasProperty(KoXmlNS::draw, property)) {
+        value = styleStack.property(KoXmlNS::draw, property);
+    }
+
+    return value;
+}
+
 
 
 // ======== KoShape
@@ -185,18 +199,18 @@ void KoShape::paintDecorations(QPainter &painter, const KoViewConverter &convert
     Q_UNUSED(converter);
     Q_UNUSED(canvas);
     /* Since this code is not actually used (kivio is going to be the main user) lets disable instead of fix.
-        if ( selected )
+        if (selected)
         {
             // draw connectors
-            QPen pen( Qt::blue );
-            pen.setWidth( 0 );
-            painter.setPen( pen );
-            painter.setBrush( Qt::NoBrush );
-            for ( int i = 0; i < d->connectors.size(); ++i )
+            QPen pen(Qt::blue);
+            pen.setWidth(0);
+            painter.setPen(pen);
+            painter.setBrush(Qt::NoBrush);
+            for (int i = 0; i < d->connectors.size(); ++i)
             {
                 QPointF p = converter.documentToView(d->connectors[ i ]);
-                painter.drawLine( QPointF( p.x() - 2, p.y() + 2 ), QPointF( p.x() + 2, p.y() - 2 ) );
-                painter.drawLine( QPointF( p.x() + 2, p.y() + 2 ), QPointF( p.x() - 2, p.y() - 2 ) );
+                painter.drawLine(QPointF(p.x() - 2, p.y() + 2), QPointF(p.x() + 2, p.y() - 2));
+                painter.drawLine(QPointF(p.x() + 2, p.y() + 2), QPointF(p.x() - 2, p.y() - 2));
             }
         }*/
 }
@@ -250,22 +264,6 @@ void KoShape::setSize(const QSizeF &newSize)
     if (oldSize == newSize)
         return;
 
-    QMatrix matrix;
-    oldSize.setHeight(qMax((qreal) 1E-4, oldSize.height())); // avoids devision by zero below
-    oldSize.setWidth(qMax((qreal) 1E-4, oldSize.width()));
-    matrix.scale(newSize.width()/oldSize.width(), newSize.height()/oldSize.height());
-
-    KoGradientBackground * g = dynamic_cast<KoGradientBackground*>(d->fill);
-    if (g) {
-        g->setMatrix(g->matrix() * matrix);
-    }
-    KoLineBorder *l = dynamic_cast<KoLineBorder*>(d->border);
-    if (l && l->lineBrush().gradient()) {
-        QBrush brush = l->lineBrush();
-        brush.setMatrix(brush.matrix() * matrix);
-        l->setLineBrush(brush);
-    }
-
     d->size = newSize;
 
     notifyChanged();
@@ -327,14 +325,14 @@ QRectF KoShape::boundingRect() const
     bb = transform.mapRect(bb);
     if (d->shadow) {
         KoInsets insets;
-        d->shadow->insets(this, insets);
+        d->shadow->insets(insets);
         bb.adjust(-insets.left, -insets.top, insets.right, insets.bottom);
     }
     if (d->filterEffectStack) {
         QRectF clipRect = d->filterEffectStack->clipRectForBoundingRect(QRectF(QPointF(), mySize));
         bb |= transform.mapRect(clipRect);
     }
-    
+
     return bb;
 }
 
@@ -514,7 +512,7 @@ void KoShape::copySettings(const KoShape *shape)
     Q_D(KoShape);
     d->size = shape->size();
     d->connectors.clear();
-    foreach(const QPointF & point, shape->connectionPoints())
+    foreach(const QPointF &point, shape->connectionPoints())
         addConnectionPoint(point);
     d->zIndex = shape->zIndex();
     d->visible = shape->isVisible();
@@ -576,8 +574,7 @@ bool KoShape::hasTransparency()
 void KoShape::setTransparency(qreal transparency)
 {
     Q_D(KoShape);
-
-    d->transparency = qBound(qreal(0.0), qreal(transparency), qreal(1.0));
+    d->transparency = qBound<qreal>(0.0, transparency, 1.0);
 }
 
 qreal KoShape::transparency(bool recursive) const
@@ -633,7 +630,7 @@ QPointF KoShape::position() const
     Q_D(const KoShape);
     QPointF center(0.5*size().width(), 0.5*size().height());
     return d->localMatrix.map(center) - center;
-    //return d->localMatrix.map( QPointF(0,0) );
+    //return d->localMatrix.map(QPointF(0,0));
 }
 
 void KoShape::addConnectionPoint(const QPointF &point)
@@ -650,7 +647,7 @@ QList<QPointF> KoShape::connectionPoints() const
     QList<QPointF> points;
     QSizeF s = size();
     // convert glue points to shape coordinates
-    foreach(const QPointF & cp, d->connectors)
+    foreach(const QPointF &cp, d->connectors)
         points.append(QPointF(s.width() * cp.x(), s.height() * cp.y()));
 
     return points;
@@ -674,14 +671,14 @@ QSet<KoEventAction *> KoShape::eventActions() const
     return d->eventActions;
 }
 
-void KoShape::setBackground(KoShapeBackground * fill)
+void KoShape::setBackground(KoShapeBackground *fill)
 {
     Q_D(KoShape);
     if (d->fill)
-        d->fill->removeUser();
+        d->fill->deref();
     d->fill = fill;
     if (d->fill)
-        d->fill->addUser();
+        d->fill->ref();
     d->shapeChanged(BackgroundChanged);
     notifyChanged();
 }
@@ -825,31 +822,31 @@ void KoShape::setBorder(KoShapeBorderModel *border)
 {
     Q_D(KoShape);
     if (border)
-        border->addUser();
+        border->ref();
     d->updateBorder();
     if (d->border)
-        d->border->removeUser();
+        d->border->deref();
     d->border = border;
     d->updateBorder();
     d->shapeChanged(BorderChanged);
     notifyChanged();
 }
 
-void KoShape::setShadow(KoShapeShadow * shadow)
+void KoShape::setShadow(KoShapeShadow *shadow)
 {
     Q_D(KoShape);
     if (d->shadow)
-        d->shadow->removeUser();
+        d->shadow->deref();
     d->shadow = shadow;
     if (d->shadow) {
-        d->shadow->addUser();
+        d->shadow->ref();
         // TODO update changed area
     }
     d->shapeChanged(ShadowChanged);
     notifyChanged();
 }
 
-KoShapeShadow * KoShape::shadow() const
+KoShapeShadow *KoShape::shadow() const
 {
     Q_D(const KoShape);
     return d->shadow;
@@ -874,7 +871,7 @@ QString KoShape::name() const
     return d->name;
 }
 
-void KoShape::setName(const QString & name)
+void KoShape::setName(const QString &name)
 {
     Q_D(KoShape);
     d->name = name;
@@ -912,23 +909,23 @@ QString KoShape::saveStyle(KoGenStyle &style, KoShapeSavingContext &context) con
 {
     Q_D(const KoShape);
     // and fill the style
-    KoShapeBorderModel * b = border();
+    KoShapeBorderModel *b = border();
     if (b) {
         b->fillStyle(style, context);
     }
     else {
-        style.addProperty( "draw:stroke", "none" );
+        style.addProperty("draw:stroke", "none");
     }
-    KoShapeShadow * s = shadow();
+    KoShapeShadow *s = shadow();
     if (s)
         s->fillStyle(style, context);
 
-    KoShapeBackground * bg = background();
+    KoShapeBackground *bg = background();
     if (bg) {
         bg->fillStyle(style, context);
     }
     else {
-        style.addProperty( "draw:fill", "none" );
+        style.addProperty("draw:fill", "none");
     }
 
     if (context.isSet(KoShapeSavingContext::AutoStyleInStyleXml)) {
@@ -951,13 +948,13 @@ QString KoShape::saveStyle(KoGenStyle &style, KoShapeSavingContext &context) con
         style.addProperty(it.key(), it.value());
     }
 
-    return context.mainStyles().lookup(style, context.isSet(KoShapeSavingContext::PresentationShape) ? "pr" : "gr");
+    return context.mainStyles().insert(style, context.isSet(KoShapeSavingContext::PresentationShape) ? "pr" : "gr");
 }
 
-void KoShape::loadStyle(const KoXmlElement & element, KoShapeLoadingContext &context)
+void KoShape::loadStyle(const KoXmlElement &element, KoShapeLoadingContext &context)
 {
     Q_D(KoShape);
-    
+
     KoStyleStack &styleStack = context.odfLoadingContext().styleStack();
     styleStack.save();
 
@@ -970,15 +967,15 @@ void KoShape::loadStyle(const KoXmlElement & element, KoShapeLoadingContext &con
         styleStack.setTypeProperties("graphic");
     }
 
-    if(d->fill && !d->fill->removeUser()) {
+    if (d->fill && !d->fill->deref()) {
         delete d->fill;
         d->fill = 0;
     }
-    if(d->border && !d->border->removeUser()) {
+    if (d->border && !d->border->deref()) {
         delete d->border;
         d->border = 0;
     }
-    if(d->shadow && !d->shadow->removeUser()) {
+    if (d->shadow && !d->shadow->deref()) {
         delete d->shadow;
         d->shadow = 0;
     }
@@ -989,7 +986,7 @@ void KoShape::loadStyle(const KoXmlElement & element, KoShapeLoadingContext &con
     styleStack.restore();
 }
 
-bool KoShape::loadOdfAttributes(const KoXmlElement & element, KoShapeLoadingContext &context, int attributes)
+bool KoShape::loadOdfAttributes(const KoXmlElement &element, KoShapeLoadingContext &context, int attributes)
 {
     Q_D(KoShape);
     if (attributes & OdfPosition) {
@@ -1012,7 +1009,7 @@ bool KoShape::loadOdfAttributes(const KoXmlElement & element, KoShapeLoadingCont
 
     if (attributes & OdfLayer) {
         if (element.hasAttributeNS(KoXmlNS::draw, "layer")) {
-            KoShapeLayer * layer = context.layer(element.attributeNS(KoXmlNS::draw, "layer"));
+            KoShapeLayer *layer = context.layer(element.attributeNS(KoXmlNS::draw, "layer"));
             if (layer) {
                 setParent(layer);
             }
@@ -1031,8 +1028,7 @@ bool KoShape::loadOdfAttributes(const KoXmlElement & element, KoShapeLoadingCont
     if (attributes & OdfZIndex) {
         if (element.hasAttributeNS(KoXmlNS::draw, "z-index")) {
             setZIndex(element.attributeNS(KoXmlNS::draw, "z-index").toInt());
-        }
-        else {
+        } else {
             setZIndex(context.zIndex());
         }
     }
@@ -1055,7 +1051,7 @@ bool KoShape::loadOdfAttributes(const KoXmlElement & element, KoShapeLoadingCont
 
     if (attributes & OdfAdditionalAttributes) {
         QSet<KoShapeLoadingContext::AdditionalAttributeData> additionalAttributeData = KoShapeLoadingContext::additionalAttributeData();
-        foreach(const KoShapeLoadingContext::AdditionalAttributeData & attributeData, additionalAttributeData) {
+        foreach(const KoShapeLoadingContext::AdditionalAttributeData &attributeData, additionalAttributeData) {
             if (element.hasAttributeNS(attributeData.ns, attributeData.tag)) {
                 QString value = element.attributeNS(attributeData.ns, attributeData.tag);
                 //kDebug(30006) << "load additional attribute" << attributeData.tag << value;
@@ -1075,34 +1071,23 @@ bool KoShape::loadOdfAttributes(const KoXmlElement & element, KoShapeLoadingCont
     return true;
 }
 
-QString KoShape::getStyleProperty(const char *property, const KoXmlElement & element, KoShapeLoadingContext & context)
+KoShapeBackground *KoShape::loadOdfFill(const KoXmlElement &element, KoShapeLoadingContext &context) const
 {
-    Q_UNUSED(element);
-    KoStyleStack &styleStack = context.odfLoadingContext().styleStack();
-    QString value;
-
-    if (styleStack.hasProperty(KoXmlNS::draw, property)) {
-        value = styleStack.property(KoXmlNS::draw, property);
+    QString fill = KoShapePrivate::getStyleProperty("fill", element, context);
+    KoShapeBackground *bg = 0;
+    if (fill == "solid" || fill == "hatch") {
+        bg = new KoColorBackground();
+    } else if (fill == "gradient") {
+        QGradient *gradient = new QLinearGradient();
+        gradient->setCoordinateMode(QGradient::ObjectBoundingMode);
+        bg = new KoGradientBackground(gradient);
+    } else if (fill == "bitmap") {
+        bg = new KoPatternBackground(context.imageCollection());
+    } else {
+        return 0;
     }
 
-    return value;
-}
-
-KoShapeBackground * KoShape::loadOdfFill(const KoXmlElement & element, KoShapeLoadingContext & context)
-{
-    QString fill = getStyleProperty("fill", element, context);
-    KoShapeBackground * bg = 0;
-    if (fill == "solid" || fill == "hatch")
-        bg = new KoColorBackground();
-    else if (fill == "gradient")
-        bg = new KoGradientBackground(new QLinearGradient());
-    else if (fill == "bitmap")
-        bg = new KoPatternBackground(context.imageCollection());
-
-    if (! bg)
-        return 0;
-
-    if (! bg->loadStyle(context.odfLoadingContext(), size())) {
+    if (!bg->loadStyle(context.odfLoadingContext(), size())) {
         delete bg;
         return 0;
     }
@@ -1110,16 +1095,16 @@ KoShapeBackground * KoShape::loadOdfFill(const KoXmlElement & element, KoShapeLo
     return bg;
 }
 
-KoShapeBorderModel * KoShape::loadOdfStroke(const KoXmlElement & element, KoShapeLoadingContext & context)
+KoShapeBorderModel *KoShape::loadOdfStroke(const KoXmlElement &element, KoShapeLoadingContext &context) const
 {
     KoStyleStack &styleStack = context.odfLoadingContext().styleStack();
     KoOdfStylesReader &stylesReader = context.odfLoadingContext().stylesReader();
 
-    QString stroke = getStyleProperty("stroke", element, context);
+    QString stroke = KoShapePrivate::getStyleProperty("stroke", element, context);
     if (stroke == "solid" || stroke == "dash") {
         QPen pen = KoOdfGraphicStyles::loadOdfStrokeStyle(styleStack, stroke, stylesReader);
 
-        KoLineBorder * border = new KoLineBorder();
+        KoLineBorder *border = new KoLineBorder();
 
         if (styleStack.hasProperty(KoXmlNS::koffice, "stroke-gradient")) {
             QString gradientName = styleStack.property(KoXmlNS::koffice, "stroke-gradient");
@@ -1142,12 +1127,12 @@ KoShapeBorderModel * KoShape::loadOdfStroke(const KoXmlElement & element, KoShap
     }
 }
 
-KoShapeShadow * KoShape::loadOdfShadow(const KoXmlElement & element, KoShapeLoadingContext & context)
+KoShapeShadow *KoShape::loadOdfShadow(const KoXmlElement &element, KoShapeLoadingContext &context) const
 {
     KoStyleStack &styleStack = context.odfLoadingContext().styleStack();
-    QString shadowStyle = getStyleProperty("shadow", element, context);
+    QString shadowStyle = KoShapePrivate::getStyleProperty("shadow", element, context);
     if (shadowStyle == "visible" || shadowStyle == "hidden") {
-        KoShapeShadow * shadow = new KoShapeShadow();
+        KoShapeShadow *shadow = new KoShapeShadow();
         QColor shadowColor(styleStack.property(KoXmlNS::draw, "shadow-color"));
         qreal offsetX = KoUnit::parseValue(styleStack.property(KoXmlNS::draw, "shadow-offset-x"));
         qreal offsetY = KoUnit::parseValue(styleStack.property(KoXmlNS::draw, "shadow-offset-y"));
@@ -1157,7 +1142,7 @@ KoShapeShadow * KoShape::loadOdfShadow(const KoXmlElement & element, KoShapeLoad
         if (! opacity.isEmpty() && opacity.right(1) == "%")
             shadowColor.setAlphaF(opacity.left(opacity.length() - 1).toFloat() / 100.0);
         shadow->setColor(shadowColor);
-        shadow->setVisibility(shadowStyle == "visible");
+        shadow->setVisible(shadowStyle == "visible");
 
         return shadow;
     }
@@ -1249,10 +1234,10 @@ void KoShape::saveOdfAttributes(KoShapeSavingContext &context, int attributes) c
         KoGenStyle style;
         // all items that should be written to 'draw:frame' and any other 'draw:' object that inherits this shape
         if (context.isSet(KoShapeSavingContext::PresentationShape)) {
-            style = KoGenStyle(KoGenStyle::StylePresentationAuto, "presentation");
+            style = KoGenStyle(KoGenStyle::PresentationAutoStyle, "presentation");
             context.xmlWriter().addAttribute("presentation:style-name", saveStyle(style, context));
         } else {
-            style = KoGenStyle(KoGenStyle::StyleGraphicAuto, "graphic");
+            style = KoGenStyle(KoGenStyle::GraphicAutoStyle, "graphic");
             context.xmlWriter().addAttribute("draw:style-name", saveStyle(style, context));
         }
     }
@@ -1343,12 +1328,6 @@ void KoShape::saveOdfCommonChildElements(KoShapeSavingContext &context) const
 
 // end loading & saving methods
 
-void KoShape::init(const QMap<QString, KoDataCenter *> & dataCenterMap)
-{
-    Q_UNUSED(dataCenterMap);
-}
-
-
 // static
 void KoShape::applyConversion(QPainter &painter, const KoViewConverter &converter)
 {
@@ -1377,7 +1356,7 @@ QRectF KoShape::documentToShape(const QRectF &rect) const
     return absoluteTransformation(0).inverted().mapRect(rect);
 }
 
-bool KoShape::addDependee(KoShape * shape)
+bool KoShape::addDependee(KoShape *shape)
 {
     Q_D(KoShape);
     if (! shape)
@@ -1393,7 +1372,7 @@ bool KoShape::addDependee(KoShape * shape)
     return true;
 }
 
-void KoShape::removeDependee(KoShape * shape)
+void KoShape::removeDependee(KoShape *shape)
 {
     Q_D(KoShape);
     int index = d->dependees.indexOf(shape);
@@ -1401,7 +1380,7 @@ void KoShape::removeDependee(KoShape * shape)
         d->dependees.removeAt(index);
 }
 
-bool KoShape::hasDependee(KoShape * shape) const
+bool KoShape::hasDependee(KoShape *shape) const
 {
     Q_D(const KoShape);
     return d->dependees.contains(shape);
@@ -1418,60 +1397,72 @@ KoSnapData KoShape::snapData() const
     return KoSnapData();
 }
 
-void KoShape::setAdditionalAttribute(const char * name, const QString & value)
+void KoShape::setAdditionalAttribute(const char *name, const QString &value)
 {
     Q_D(KoShape);
     d->additionalAttributes.insert(name, value);
 }
 
-void KoShape::removeAdditionalAttribute(const char * name)
+void KoShape::removeAdditionalAttribute(const char *name)
 {
     Q_D(KoShape);
     d->additionalAttributes.remove(name);
 }
 
-bool KoShape::hasAdditionalAttribute(const char * name) const
+bool KoShape::hasAdditionalAttribute(const char *name) const
 {
     Q_D(const KoShape);
     return d->additionalAttributes.contains(name);
 }
 
-QString KoShape::additionalAttribute(const char * name) const
+QString KoShape::additionalAttribute(const char *name) const
 {
     Q_D(const KoShape);
     return d->additionalAttributes.value(name);
 }
 
-void KoShape::setAdditionalStyleAttribute(const char * name, const QString & value)
+void KoShape::setAdditionalStyleAttribute(const char *name, const QString &value)
 {
     Q_D(KoShape);
     d->additionalStyleAttributes.insert(name, value);
 }
 
-void KoShape::removeAdditionalStyleAttribute(const char * name)
+void KoShape::removeAdditionalStyleAttribute(const char *name)
 {
     Q_D(KoShape);
     d->additionalStyleAttributes.remove(name);
 }
 
-KoFilterEffectStack * KoShape::filterEffectStack() const
+KoFilterEffectStack *KoShape::filterEffectStack() const
 {
     Q_D(const KoShape);
     return d->filterEffectStack;
 }
 
-void KoShape::setFilterEffectStack(KoFilterEffectStack * filterEffectStack)
+void KoShape::setFilterEffectStack(KoFilterEffectStack *filterEffectStack)
 {
     Q_D(KoShape);
     if (d->filterEffectStack)
-        d->filterEffectStack->removeUser();
+        d->filterEffectStack->deref();
     d->filterEffectStack = filterEffectStack;
     if (d->filterEffectStack) {
-        d->filterEffectStack->addUser();
+        d->filterEffectStack->ref();
     }
     notifyChanged();
 }
 
+QSet<KoShape*> KoShape::toolDelegates() const
+{
+    Q_D(const KoShape);
+    return d->toolDelegates;
+}
+
+void KoShape::setToolDelegates(const QSet<KoShape*> &delegates)
+{
+    Q_D(KoShape);
+    d->toolDelegates = delegates;
+}
+
 KoShapePrivate *KoShape::priv()
 {
     Q_D(KoShape);
